#!/usr/bin/perl

############################################
##                                        ##
##           WebAdverts (Admin)           ##
##           by Darryl Burgdorf           ##
##       (e-mail burgdorf@awsd.com)       ##
##                                        ##
##         last modified:  9/7/97         ##
##           copyright (c) 1997           ##
##                                        ##
##    latest version is available from    ##
##        http://awsd.com/scripts/        ##
##                                        ##
############################################

# COPYRIGHT NOTICE:
#
# Copyright 1997 Darryl C. Burgdorf.  All Rights Reserved.
#
# This program is being distributed as shareware.  It may be used and
# modified by anyone, so long as this copyright notice and the header
# above remain intact, but any commercial use should be registered.  By
# using this program you agree to indemnify Darryl C. Burgdorf from any
# liability.
#
# Selling the code for this program without prior written consent is
# expressly forbidden.  Obtain permission before redistributing this
# program over the Internet or in any other medium.  In all cases
# copyright and header must remain intact.

# VERSION HISTORY:
#
# 1.21  09/07/97  Fixed "fall-thru after click-thru" problem
#                 Fixed problem with displays not giving exposures
#                 Improved handling of "bad" click-thrus
#                 Reorganized main advert edit screen
#                 Removed SSI elements from admin banner displays
#                 Added zone to info maintained in non-SSI log file
#                 Squashed "text reverts to bottom" bug
#                 "Dealt with" extra (blank) adcount.txt lines
#                 Various minor tweaks
# 1.20  08/31/97  Added non-SSI (banner exchange) option
#                 Added basing of exposures on banner exchange "ratio"
#                 Further simplified config files
#                 Made backups optional
#                 Added optional flock() control of semaphore files
#                 Fixed minor typo in banner display
#                 Fixed lock-related typos in "gotoad" subroutine
# 1.14  08/25/97  Rewrote file locking & backup "from the ground up"
#                   (Replaced flock() system with semaphore system)
#                 Abbreviated some table headings
#                 Closed minor security hole
# 1.13  08/17/97  Fixed bug in initialization of group list
#                 Revised backup system
# 1.12  08/16/97  Added logging of daily stats
#                 Allowed for each zone to cycle distinctly
#                 Added ability to "group" adverts
#                 Removed link texts from "show all" display
#                 Changed TARGET def to general link attribute def
#                 Added auto backups as supplement to file locking
#                 "Cleaned up" initial password entry
#                 Revised "look" of admin pages
#                 Improved parsing of URLs
#                 Really fixed "BORDER=0" & "WEIGHT=0" bugs
# 1.11  07/26/97  Fixed "exclusive access/read only" lock bug
#                 Fixed bug causing display script to "lose" extra URLs
#                 Fixed "BORDER=0" bug
#                 Fixed minor time display-related bugs
#                 Changed URL checking to allow commas
# 1.10  07/19/97  FIRST SHAREWARE RELEASE
#                 Consolidated various display scripts into one
#                 Added ability to "zone" adverts via separate configs
#                 Revamped "weighting" system from scratch
#                 Allowed definition of individual expiration criteria
#                 Allowed for multiple banners in a single advert
#                 Made URL, link text, height and width optional
#                 Added ability to put text above *or* below banner
#                 Allowed designation of TARGET and BORDER attributes
#                 Added "raw" HTML banner code option
#                 Fixed minor bugs in total exposure count
#                 Trapped for bad characters in new advert names
#                 Trapped for previously-problematical commas
#                 Moved "delete" from main page to advert edit page
#                 Improved handling of administrative password
#                 Lots of other miscellaneous small improvements
# 1.04  05/27/97  Added new script to display all active banners
#                 Put variable assignments in configuration script
#                 Added option to define max click-thrus
#                 Added ability to "weight" banners
#                 Added start date to log info for each advert
#                 Added "exposures per day" to advert info display
#                 Improved error handling
#                 Added basic URL format validation
#                 Numerous "tweaks" and format adjustments
# 1.03  05/22/97  Added configurable link text
#                 Added new script to put banners in CGI pages
#                 Fixed bugs introduced by previous file locking "fix"
#                 Corrected style of main password input box
#                 Minor bug fixes in display code
# 1.02  04/20/97  Fixed file locking!
#                 Added actual click-thru ratios to display
#                 Revised display of percentages
# 1.01  04/03/97  Allowed for automatic creation of necessary text files
#                 Closed hole that allowed spaces in file names
# 1.00  04/01/97  Initial "public" release

# The following variables should be set to define the locations
# and URLs of various files, as explained in the documentation.

$adverts_dir = "/full/path/to/ads";
$admin_cgi = "http://foo.com/ads/ads_admin.pl";
$nonssi_cgi = "http://foo.com/ads/ads_nonssi.pl";

$UseBackup = 1;
$UseLocking = 1;

# NOTHING BELOW THIS LINE NEEDS TO BE ALTERED!

@months = (Jan,Feb,Mar,Apr,May,Jun,Jul,Aug,Sep,Oct,Nov,Dec);
$version = "1.21";

print "Content-type: text/html\n\n";

read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
@pairs = split(/&/, $buffer);
foreach $pair (@pairs) {
	($name, $value) = split(/=/, $pair);
	$value =~ tr/+/ /;
	$value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
	if ($INPUT{$name}) { $INPUT{$name} = $INPUT{$name}.",".$value; }
	else { $INPUT{$name} = $value; }
}

$time = time;

if ($ENV{'QUERY_STRING'} =~ /reginfo/) { &reginfo; }

if ($INPUT{'register'}) { &register; }
elsif ($INPUT{'edit'}) { &edit; }
elsif ($INPUT{'groupedit'}) { &groupedit; }
elsif ($INPUT{'del'}) { &del; }
elsif ($INPUT{'delgroup'}) { &delgroup; }
elsif ($INPUT{'newpass'}) { &newpass; }
elsif ($INPUT{'resetcount'}) { &resetcount; }
elsif ($INPUT{'editfinal'}) { &editfinal; }
elsif ($INPUT{'editgroupfinal'}) { &editgroupfinal; }
elsif ($INPUT{'delfinal'}) { &delfinal; }
elsif ($INPUT{'delgroupfinal'}) { &delgroupfinal; }
elsif ($INPUT{'reviewone'} eq "Review All Adverts") { &reviewall; }
elsif ($INPUT{'reviewone'} =~ s/^G //) { &reviewgroup; }
elsif ($INPUT{'reviewone'}) { &reviewone; }
elsif ($INPUT{'dailystats'}) { &dailystats; }
else { &printoptions; }

sub printoptions {
	$lockerror = &LockFile("adpassword.txt");
	if ($lockerror) {
		&Error_File("adpassword.txt");
		exit;
	}
	open (PASSWORD, "$adverts_dir/adpassword.txt");
	$password = <PASSWORD>;
	close (PASSWORD);
	&UnlockFile("adpassword.txt");
	chop ($password) if ($password =~ /\n$/);
	if (!$password) {
		&InitializePassword;
	}
	&Header("WebAdverts","WebAdverts Administrative Access");
	print "<P>To view the status of all adverts and access ";
	print "WebAdvert's main administrative functions, ";
	print "input the administrative password and select ";
	print "&quot;Review All Adverts.&quot; To view the status ";
	print "of any single advert or defined group, ";
	print "simply select it and enter its password.</P>\n";
	print "<CENTER><FORM METHOD=POST ACTION=$admin_cgi>\n";
	print "<P><STRONG>Password:</STRONG> ";
	print "<INPUT TYPE=PASSWORD NAME=password SIZE=15>\n";
	print "<P><SELECT NAME=reviewone>";
	print "<OPTION VALUE=\"Review All Adverts\" ";
	print "SELECTED> Review All Adverts ";
	if (-e "$adverts_dir/groups.txt") {
		$lockerror = &LockFile("groups.txt");
		if ($lockerror) {
			undef (@lines);
		}
		else {
			open (COUNT, "+<$adverts_dir/groups.txt");
			@lines = <COUNT>;
			close (COUNT);
			&UnlockFile("groups.txt");
		}
		foreach $line (@lines) { chop ($line) if ($line =~ /\n$/); }
		foreach $group (@lines) {
			print "<OPTION VALUE=\"G $group\"> ";
			print "Review Group: $group ";
		}
	}
	if (-e "$adverts_dir/adcount.txt") {
		$lockerror = &LockFile("adcount.txt");
		if ($lockerror) {
			undef (@lines);
		}
		else {
			open (COUNT, "+<$adverts_dir/adcount.txt");
			@lines = <COUNT>;
			close (COUNT);
			&UnlockFile("adcount.txt");
		}
		foreach $line (@lines) { chop ($line) if ($line =~ /\n$/); }
		$max = @lines - 1;
		@advertisements = @lines[3..$max];
		foreach $advertiser (@advertisements) {
			next if (length($advertiser) < 1);
			print "<OPTION VALUE=\"$advertiser\"> ";
			print "Review Advert: $advertiser ";
		}
	}
	print "</SELECT>\n";
	print "<P><INPUT TYPE=SUBMIT ";
	print "VALUE=\"Review Advert(s)\"> ";
	print "</P></FORM></CENTER>\n";
	&Footer;
	exit;
}

sub InitializePassword {
	&Header("WebAdverts","Enter an Administrative Password!");
	print "<P>Before you can do anything else, ";
	print "you'll need to set your administrative password. ";
	print "This will allow you to access the admin functions, ";
	print "create and edit adverts, review statistics, etc. ";
	print "Please enter your desired password below. ";
	print "(Enter it twice.)\n";
	print "<FORM METHOD=POST ACTION=$admin_cgi>\n";
	print "<P><CENTER><INPUT TYPE=SUBMIT NAME=newpass ";
	print "VALUE=\"Set Admin Password:\"> ";
	print "<INPUT TYPE=PASSWORD NAME=passad SIZE=10> ";
	print "<INPUT TYPE=PASSWORD NAME=passad2 SIZE=10>\n";
	print "</CENTER></P></FORM>\n";
	&Footer;
	exit;
}

sub reviewall {
	unless ($actionresult == 1) {
		$lockerror = &LockFile("adpassword.txt");
		if ($lockerror) {
			&Error_File("adpassword.txt");
			exit;
		}
		open (PASSWORD, "$adverts_dir/adpassword.txt");
		$password = <PASSWORD>;
		close (PASSWORD);
		&UnlockFile("adpassword.txt");
		chop ($password) if ($password =~ /\n$/);
		if (!$password) {
			&InitializePassword;
		}
		if ($INPUT{'cryptword'}) {
			$newpassword = $INPUT{'cryptword'};
		}
		elsif ($INPUT{'password'}) {
			$newpassword = crypt($INPUT{'password'}, aa);
		}
		else {
			&Header("WebAdverts Error!","No Password!");
			print "<P ALIGN=CENTER>You must ";
			print "enter a password!</P>\n";
			&Footer;
			exit;
		}
		unless ($newpassword eq $password) {
			&Header("WebAdverts Error!","Invalid Password!");
			print "<P ALIGN=CENTER>";
			print "The password you entered is incorrect!</P>\n";
			&Footer;
			exit;
		}
	}
	&Header("WebAdverts","WebAdverts Administrative Display");
	unless (-e "$adverts_dir/register.txt") {
		print "<P ALIGN=CENTER><STRONG>";
		print "Unregistered copy.</STRONG> ";
		print "<A HREF=\"$admin_cgi?reginfo\">Click here</A> ";
		print "for registration info.</P><HR>\n";
	}
	print "<P ALIGN=CENTER><BIG><BIG><STRONG>The Following Adverts ";
	print "Are Running:</STRONG></BIG></BIG>\n";
	print "<P><CENTER><TABLE CELLPADDING=3>\n";
	print "<TR ALIGN=CENTER VALIGN=BOTTOM>";
	print "<TD><EM>Advert</EM><BR><HR></TD>";
	print "<TD><EM>Start</EM><BR><HR></TD>";
	print "<TD><EM>End</EM><BR><HR></TD>";
	print "<TD><EM>Zone(s)</EM><BR><HR></TD>";
	print "<TD><EM>Wt.</EM><BR><HR></TD>";
	print "<TD><EM>Exposures</EM><BR><HR></TD>";
	print "<TD><EM>Exp./Day</EM><BR><HR></TD>";
	print "<TD><EM>Clicks</EM><BR><HR></TD>";
	print "<TD><EM>%</EM><BR><HR></TD>";
	print "<TD><EM>Ratio</EM><BR><HR></TD></TR>\n";
	$lockerror = &LockFile("adcount.txt");
	if ($lockerror) {
		print "</TABLE></CENTER></P>\n";
		print "<P ALIGN=CENTER>[ File Error: ";
		print "adcount.txt ]";
		print "</P></FORM>\n";
		&Footer;
		exit;
	}
	if (-e "$adverts_dir/adcount.txt") {
		open (COUNT, "+<$adverts_dir/adcount.txt");
	}
	else {
		open (COUNT, ">$adverts_dir/adcount.txt");
	}
	@lines = <COUNT>;
	if (@lines < 2) {
		$lines[0] = 1;
		$lines[1] = "0";
		$lines[2] = time;
		seek(COUNT, 0, 0);
		foreach $line (@lines) { print COUNT "$line\n"; }
		truncate (COUNT, tell(COUNT));
	}
	close (COUNT);
	&UnlockFile("adcount.txt");
	foreach $line (@lines) { chop ($line) if ($line =~ /\n$/); }
	$max = @lines - 1;
	$exposures = $lines[1];
	($exposures,$cycles) = split (/\|/, $exposures);
	$starttime = $lines[2];
	@advertisements = @lines[3..$max];
	foreach $advertiser (@advertisements) {
		next if (length($advertiser) < 1);
		$lockerror = &LockFile("$advertiser.txt");
		if ($lockerror) {
			print "<TR ALIGN=CENTER><TD COLSPAN=10>";
			print "[ File Error: ";
			print "$advertiser.txt ]</TD></TR>\n";
			next;
		}
		open (DISPLAY, "+<$adverts_dir/$advertiser.txt");
		@lines = <DISPLAY>;
		close (DISPLAY);
		&UnlockFile("$advertiser.txt");
		foreach $line (@lines) { chop ($line) if ($line =~ /\n$/); }
		($max,$shown,$visits,$url,$image,$height,$width,
		  $alt,$pass,$text,$start,$weight,$zone,
		  $border,$target,$raw,$displayratio) = @lines;
		unless ($max || $displayratio) {
			print "<TR ALIGN=CENTER><TD COLSPAN=10>";
			print "[ File Error: ";
			print "$advertiser.txt ]</TD></TR>\n";
			next;
		}
		($max,$maxtype) = split (/\|/, $max);
		unless ($maxtype) { $maxtype = "E"; }
		($text,$texttype) = split (/\|/, $text);
		unless ($texttype) { $texttype = "B"; }
		($displayratio,$displaycount) = split (/\|/, $displayratio);
		if (($maxtype eq "E") && ($displayratio > 0)) {
			$max = $max+int($displaycount/$displayratio);
		}
		if ($shown == 0) {
			$perc = "N/A";
			$ratio = "N/A";
		}
		elsif ($visits == 0) {
			$perc = "N/A";
			$ratio = "N/A";
		}
		else {
			$perc = ((100*($visits/$shown))+.05001);
			$ratio = (($shown/$visits)+.5001);
		}
		unless ($perc eq "N/A") {
			$perc =~ s/(\d+\.\d).*/$1/;
			$perc = $perc."%";
		}
		unless ($ratio eq "N/A") {
			$ratio =~ s/(\d+)\.\d.*/$1/;
			$ratio = $ratio.":1";
		}
		print "<TR ALIGN=CENTER>";
		print "<TD><STRONG>$advertiser</STRONG></TD>";
		$runtime = 0;
		if ($start) {
			($sec,$min,$hour,$mday,$mon,$year,
			  $wday,$yday,$isdst) = localtime($start);
			print "<TD>$mday @months[$mon]</TD>";
			$runtime = $time - $start + 1;
		}
		else { print "<TD></TD>"; }
		if ($maxtype eq "D") {
			($sec,$min,$hour,$mday,$mon,$year,
			  $wday,$yday,$isdst) = localtime($max);
			print "<TD>$mday @months[$mon]";
			unless ($max > $time) {
				print "<BR><EM>EXPIRED!</EM>";
			}
			print "</TD>";
		}
		else {
			print "<TD>",&commas($max);
			if ($maxtype eq "C") {
				print " clicks";
				unless ($max > $visits) {
					print "<BR><EM>EXPIRED!</EM>";
				}
			}
			else {
				print " exposures";
				unless (($max > $shown) || ($displayratio > 0)) {
					print "<BR><EM>EXPIRED!</EM>";
				}
			}
			print "</TD>";
		}
		print "<TD>$zone</TD>";
		print "<TD>$weight</TD>";
		print "<TD>",&commas($shown),"</TD>";
		if ($runtime > 86400) {
			$average = int(($shown/($runtime/86400))+.5);
			print "<TD>",&commas($average),"</TD>";
		}
		else {
			print "<TD>N/A</TD>";
		}
		print "<TD>",&commas($visits),"</TD>";
		print "<TD>$perc</TD><TD>$ratio</TD></TR>\n";
	}
	print "<TR ALIGN=CENTER>";
	print "<TD COLSPAN=10>";
	($sec,$min,$hour,$mday,$mon,$year,
	  $wday,$yday,$isdst) = localtime($time);
	if ($hour < 10) { $hour = "0".$hour; }
	if ($min < 10) { $min = "0".$min; }
	print "(These figures are accurate as of $hour:$min on $mday @months[$mon] 19$year.";
	($sec,$min,$hour,$mday,$mon,$year,
	  $wday,$yday,$isdst) = localtime($starttime);
	if ($hour < 10) { $hour = "0".$hour; }
	if ($min < 10) { $min = "0".$min; }
	print "<BR>Since $hour:$min on $mday @months[$mon] 19$year, ";
	print "there have been a total of <STRONG>",&commas($exposures);
	print "</STRONG> advert exposures";
	$time = $time - $starttime + 1;
	if ($time > 86400) {
		$average = int(($exposures/($time/86400))+.5);
		print ",<BR>for an average of <STRONG>",&commas($average);
		print "</STRONG> exposures per day";
	}
	print ".)</TD></TR>\n";
	print "</TABLE></CENTER></P>\n";
	if (-e "$adverts_dir/groups.txt") {
		$lockerror = &LockFile("groups.txt");
		if ($lockerror) {
			undef @grouplines;
		}
		else {
			open (COUNT, "+<$adverts_dir/groups.txt");
			@grouplines = <COUNT>;
			close (COUNT);
			&UnlockFile("groups.txt");
		}
	}
	if (@grouplines > 0) {
		print "<HR><P ALIGN=CENTER><BIG><BIG><STRONG>";
		print "The Following Groups Have Been Defined:";
		print "</STRONG></BIG></BIG>\n";
		foreach $group (@grouplines) {
			chop ($group) if ($group =~ /\n$/);
			next if (length($group) < 1);
			print "<P ALIGN=CENTER>";
			print "<STRONG>$group:</STRONG>";
			$lockerror = &LockFile("$group.grp");
			if ($lockerror) {
				print " [ File Error ]";
			}
			else {
				open (DISPLAY, "+<$adverts_dir/$group.grp");
				@members = <DISPLAY>;
				close (DISPLAY);
				&UnlockFile("$group.grp");
				$password = @members[0];
				foreach $member (@members) {
					unless ($member eq $password) {
						chop ($member)
						  if ($member =~ /\n$/);
						print " $member";
					}
				}
			}
			print "\n";
		}
	}
	print "<HR><FORM METHOD=POST ACTION=$admin_cgi>\n";
	$lockerror = &LockFile("adpassword.txt");
	if ($lockerror) {
		&Error_File("adpassword.txt");
		exit;
	}
	open (PASSWORD, "$adverts_dir/adpassword.txt");
	$password = <PASSWORD>;
	close (PASSWORD);
	&UnlockFile("adpassword.txt");
	chop ($password) if ($password =~ /\n$/);
	print "<INPUT TYPE=HIDDEN NAME=cryptword ";
	print "VALUE=\"$password\">\n";
	print "<P ALIGN=CENTER><BIG><BIG><STRONG>The Following Options ";
	print "Are Available:</STRONG></BIG></BIG>\n";
	print "<P><CENTER><INPUT TYPE=SUBMIT NAME=edit ";
	print "VALUE=\"Add/Edit/Delete Advert:\"> ";
	print "<INPUT TYPE=TEXT NAME=editad SIZE=25 ";
	print "VALUE=\"(advert name)\">\n";
	print "<P><CENTER><INPUT TYPE=SUBMIT NAME=groupedit ";
	print "VALUE=\"Add/Edit/Delete Group:\"> ";
	print "<INPUT TYPE=TEXT NAME=editgroup SIZE=25 ";
	print "VALUE=\"(group name)\">\n";
	print "<P><INPUT TYPE=SUBMIT NAME=newpass ";
	print "VALUE=\"Change Admin Password:\"> ";
	print "<INPUT TYPE=PASSWORD NAME=passad SIZE=10> ";
	print "<INPUT TYPE=PASSWORD NAME=passad2 SIZE=10>\n";
	print "<P><INPUT TYPE=SUBMIT NAME=resetcount ";
	print "VALUE=\"Reset Overall Total Exposures Count\">\n";
	print "</P></CENTER></FORM>\n";
	&LinkBack;
	&Footer;
	exit;
}

sub reviewgroup {
	$groupstatus = "G $INPUT{'reviewone'}";
	unless ($INPUT{'password'}) {
		&Header("WebAdverts Error!","No Password!");
		print "<P ALIGN=CENTER>You must enter a password!</P>\n";
		&Footer;
		exit;
	}
	unless (-e "$adverts_dir/$INPUT{'reviewone'}.grp") {
		&Header("WebAdverts Error!","Invalid Group Name!");
		print "<P ALIGN=CENTER>There is no defined group ";
		print "with the name ";
		print "<STRONG>&quot;$INPUT{'reviewone'}&quot;</STRONG> ";
		print "on the list!\n";
		print "<P ALIGN=CENTER>(Note that group names ";
		print "<EM>are</EM> case sensitive!)</P>\n";
		&Footer;
		exit;
	}
	$lockerror = &LockFile("$INPUT{'reviewone'}.grp");
	if ($lockerror) {
		&Error_File("$INPUT{'reviewone'}.grp");
		exit;
	}
	open (DISPLAY, "+<$adverts_dir/$INPUT{'reviewone'}.grp");
	@adverts = <DISPLAY>;
	close (DISPLAY);
	&UnlockFile("$INPUT{'reviewone'}.grp");
	foreach $line (@adverts) { chop ($line) if ($line =~ /\n$/); }
	unless ($INPUT{'password'} eq @adverts[0]) {
		$lockerror = &LockFile("adpassword.txt");
		if ($lockerror) {
			&Error_File("adpassword.txt");
			exit;
		}
		open (PASSWORD, "$adverts_dir/adpassword.txt");
		$password = <PASSWORD>;
		close (PASSWORD);
		&UnlockFile("adpassword.txt");
		chop ($password) if ($password =~ /\n$/);
		$newpassword = crypt($INPUT{'password'}, aa);
		unless ($password && ($newpassword eq $password)) {
			&Header("WebAdverts Error!","Invalid Password!");
			print "<P ALIGN=CENTER>";
			print "The password you entered is incorrect!</P>\n";
			&Footer;
			exit;
		}
	}
	&Header("WebAdverts","WebAdverts Administrative Display");
	print "<P ALIGN=CENTER><BIG><STRONG>(Adverts in the ";
	print "<EM>$INPUT{'reviewone'}</EM> Group)</STRONG></BIG></P>\n";
	foreach $advert (@adverts) {
		$name = $advert;
		next unless (-e "$adverts_dir/$advert.txt");
		$lockerror = &LockFile("$advert.txt");
		if ($lockerror) { next; }
		open (DISPLAY, "+<$adverts_dir/$advert.txt");
		@lines = <DISPLAY>;
		close (DISPLAY);
		&UnlockFile("$advert.txt");
		foreach $line (@lines) { chop ($line) if ($line =~ /\n$/); }
		($max,$shown,$visits,$url,$image,$height,$width,
		  $alt,$pass,$text,$start,$weight,$zone,
		  $border,$target,$raw,$displayratio) = @lines;
		($max,$maxtype) = split (/\|/, $max);
		unless ($maxtype) { $maxtype = "E"; }
		($text,$texttype) = split (/\|/, $text);
		unless ($texttype) { $texttype = "B"; }
		($displayratio,$displaycount) = split (/\|/, $displayratio);
		if (($maxtype eq "E") && ($displayratio > 0)) {
			$max = $max+int($displaycount/$displayratio);
		}
		$TotalShown += $shown;
		$TotalVisits += $visits;
		print "<HR>\n";
		&reviewadvert;
	}
	print "<HR><P ALIGN=CENTER><BIG><BIG><STRONG>Master Overview:";
	print "</STRONG></BIG></BIG>\n";
	print "<P><CENTER><TABLE CELLPADDING=3>\n";
	print "<TR ALIGN=CENTER VALIGN=BOTTOM>";
	print "<TD><EM>Exposures</EM><BR><HR></TD>";
	print "<TD><EM>Clicks</EM><BR><HR></TD>";
	print "<TD><EM>%</EM><BR><HR></TD>";
	print "<TD><EM>Ratio</EM><BR><HR></TD></TR>\n";
	if ($TotalShown == 0) {
		$perc = "N/A";
		$ratio = "N/A";
	}
	elsif ($TotalVisits == 0) {
		$perc = "N/A";
		$ratio = "N/A";
	}
	else {
		$perc = ((100*($TotalVisits/$TotalShown))+.05001);
		$ratio = (($TotalShown/$TotalVisits)+.5001);
	}
	unless ($perc eq "N/A") {
		$perc =~ s/(\d+\.\d).*/$1/;
		$perc = $perc."%";
	}
	unless ($ratio eq "N/A") {
		$ratio =~ s/(\d+)\.\d.*/$1/;
		$ratio = $ratio.":1";
	}
	print "<TR ALIGN=CENTER>";
	print "<TD>",&commas($TotalShown),"</TD>";
	print "<TD>",&commas($TotalVisits),"</TD>";
	print "<TD>$perc</TD><TD>$ratio</TD></TR>\n";
	print "</TABLE></CENTER></P>\n";
	&Footer;
	exit;
}

sub reviewone {
	unless ($INPUT{'password'}) {
		&Header("WebAdverts Error!","No Password!");
		print "<P ALIGN=CENTER>You must enter a password!</P>\n";
		&Footer;
		exit;
	}
	unless (-e "$adverts_dir/$INPUT{'reviewone'}.txt") {
		&Header("WebAdverts Error!","Invalid Advert Name!");
		print "<P ALIGN=CENTER>There is no advert with the name ";
		print "<STRONG>&quot;$INPUT{'reviewone'}&quot;</STRONG> ";
		print "on the list!\n";
		print "<P ALIGN=CENTER>(Note that advert names ";
		print "<EM>are</EM> case sensitive!)</P>\n";
		&Footer;
		exit;
	}
	$name = $INPUT{'reviewone'};
	$lockerror = &LockFile("$INPUT{'reviewone'}.txt");
	if ($lockerror) {
		&Error_File("$INPUT{'reviewone'}.txt");
		exit;
	}
	open (DISPLAY, "+<$adverts_dir/$INPUT{'reviewone'}.txt");
	@lines = <DISPLAY>;
	close (DISPLAY);
	&UnlockFile("$INPUT{'reviewone'}.txt");
	foreach $line (@lines) { chop ($line) if ($line =~ /\n$/); }
	($max,$shown,$visits,$url,$image,$height,$width,
	  $alt,$pass,$text,$start,$weight,$zone,
	  $border,$target,$raw,$displayratio) = @lines;
	($max,$maxtype) = split (/\|/, $max);
	unless ($maxtype) { $maxtype = "E"; }
	($text,$texttype) = split (/\|/, $text);
	unless ($texttype) { $texttype = "B"; }
	($displayratio,$displaycount) = split (/\|/, $displayratio);
	if (($maxtype eq "E") && ($displayratio > 0)) {
		$max = $max+int($displaycount/$displayratio);
	}
	unless ($INPUT{'password'} eq $pass) {
		$lockerror = &LockFile("adpassword.txt");
		if ($lockerror) {
			&Error_File("adpassword.txt");
			exit;
		}
		open (PASSWORD, "$adverts_dir/adpassword.txt");
		$password = <PASSWORD>;
		close (PASSWORD);
		&UnlockFile("adpassword.txt");
		chop ($password) if ($password =~ /\n$/);
		$newpassword = crypt($INPUT{'password'}, aa);
		unless ($password && ($newpassword eq $password)) {
			&Header("WebAdverts Error!","Invalid Password!");
			print "<P ALIGN=CENTER>";
			print "The password you entered is incorrect!</P>\n";
			&Footer;
			exit;
		}
	}
	&Header("WebAdverts","WebAdverts Administrative Display");
	&reviewadvert;
	&Footer;
	exit;
}

sub reviewadvert {
	print "<P ALIGN=CENTER><BIG><BIG><STRONG>Current Status of the ";
	print "<EM>$name</EM> Advert:</STRONG></BIG></BIG>\n";
	print "<P><CENTER><TABLE CELLPADDING=3>\n";
	print "<TR ALIGN=CENTER VALIGN=BOTTOM>";
	print "<TD><EM>Start</EM><BR><HR></TD>";
	print "<TD><EM>End</EM><BR><HR></TD>";
	print "<TD><EM>Exposures</EM><BR><HR></TD>";
	print "<TD><EM>Exp./Day</EM><BR><HR></TD>";
	print "<TD><EM>Clicks</EM><BR><HR></TD>";
	print "<TD><EM>%</EM><BR><HR></TD>";
	print "<TD><EM>Ratio</EM><BR><HR></TD></TR>\n";
	if ($shown == 0) {
		$perc = "N/A";
		$ratio = "N/A";
	}
	elsif ($visits == 0) {
		$perc = "N/A";
		$ratio = "N/A";
	}
	else {
		$perc = ((100*($visits/$shown))+.05001);
		$ratio = (($shown/$visits)+.5001);
	}
	unless ($perc eq "N/A") {
		$perc =~ s/(\d+\.\d).*/$1/;
		$perc = $perc."%";
	}
	unless ($ratio eq "N/A") {
		$ratio =~ s/(\d+)\.\d.*/$1/;
		$ratio = $ratio.":1";
	}
	print "<TR ALIGN=CENTER>";
	$runtime = 0;
	if ($start) {
		($sec,$min,$hour,$mday,$mon,$year,
		  $wday,$yday,$isdst) = localtime($start);
		print "<TD>$mday @months[$mon]</TD>";
		$runtime = $time - $start + 1;
	}
	else { print "<TD></TD>"; }
	if ($maxtype eq "D") {
		($sec,$min,$hour,$mday,$mon,$year,
		  $wday,$yday,$isdst) = localtime($max);
		print "<TD>$mday @months[$mon]";
		unless ($max > $time) {
			print "<BR><EM>EXPIRED!</EM>";
		}
		print "</TD>";
	}
	else {
		print "<TD>",&commas($max);
		if ($maxtype eq "C") {
			print " clicks";
			unless ($max > $visits) {
				print "<BR><EM>EXPIRED!</EM>";
			}
		}
		else {
			print " exposures";
			unless (($max > $shown) || ($displayratio > 0)) {
				print "<BR><EM>EXPIRED!</EM>";
			}
		}
		print "</TD>";
	}
	print "<TD>",&commas($shown),"</TD>";
	if ($runtime > 86400) {
		$average = int(($shown/($runtime/86400))+.5);
		print "<TD>",&commas($average),"</TD>";
	}
	else {
		print "<TD>N/A</TD>";
	}
	print "<TD>",&commas($visits),"</TD>";
	print "<TD>$perc</TD><TD>$ratio</TD></TR>\n";
	print "</TABLE></CENTER></P>\n";
	if ($displayratio > 0) {
		$earnings = int($displaycount/$displayratio);
		if ($displaycount<1) { $displaycount = "0"; }
		print "<P>To date, you have displayed <STRONG>";
		print &commas($displaycount),"</STRONG> banners ";
		print "on your site, earning <STRONG>";
		print &commas($earnings),"</STRONG> exposures ";
		print "for your own advert on other sites. ";
		print "(You earn an exposure for each ";
		print "<STRONG>$displayratio</STRONG> display";
		if ($displayratio > 1) { print "s"; }
		print ".)\n";
		print "<P>The HTML code on your pages should look similar ";
		print "to the following. (It's OK to alter the general ";
		print "appearance to match the &quot;look&quot; of your ";
		print "site, but don't change the addresses!)\n";
		print "<P><STRONG>&lt;P ALIGN=CENTER&gt;&lt;A ";
		print "HREF=&quot;$nonssi_cgi?advert=NonSSI&quot;&gt;";
		print "&lt;IMG SRC=&quot;$nonssi_cgi?";
		print "ID=$name&quot;&gt;&lt;/A&gt;</STRONG>";
	}
	print "<P><CENTER><FORM METHOD=POST ACTION=$admin_cgi>\n";
	print "<INPUT TYPE=HIDDEN NAME=password ";
	print "VALUE=$INPUT{'password'}>\n";
	if ($groupstatus) {
		print "<INPUT TYPE=HIDDEN NAME=groupstatus ";
		print "VALUE=\"$groupstatus\">\n";
	}
	print "<INPUT TYPE=HIDDEN NAME=advert VALUE=$name>\n";
	print "<INPUT TYPE=SUBMIT NAME=dailystats ";
	print "VALUE=\"View Daily Stats\">";
	print "</P></FORM></CENTER>\n";
	&ShowAdvert;
}

sub dailystats {
	$lockerror = &LockFile("$INPUT{'advert'}.log");
	if ($lockerror) {
		&Header("WebAdverts Error!","No Daily Stats!");
		print "<P ALIGN=CENTER>";
		print "Sorry, but it seems ";
		print "there is no daily log file available ";
		print "for the $INPUT{'advert'} advert!</P>\n";
		&Footer;
		exit;
	}
	open (DISPLAY, "+<$adverts_dir/$INPUT{'advert'}.log");
	@lines = <DISPLAY>;
	close (DISPLAY);
	&UnlockFile("$INPUT{'advert'}.log");
	foreach $line (@lines) {
		($acc,$logstring) = ($line =~
		  /^(\d\d\d\d\d\d\d\d\d\d) (\d\d \d\d \d\d\d\d \w)$/);
		$accesses{$logstring} = int($acc);
		($mday,$mon,$year,$type) = ($logstring =~
			  /^(\d+) (\d+) (\d+) (\w)/);
		if ($type eq "S") { $bannex = 1; }
		unless ($startday) {
			&date_to_count(int($mon),int($mday),($year-1900));
			$startday = $perp_days;
		}
	}
	&date_to_count(int($mon),int($mday),($year-1900));
	$endday = $perp_days;
	&Header("WebAdverts","WebAdverts Administrative Display");
	print "<P ALIGN=CENTER><BIG><BIG><STRONG>Daily Stats for the ";
	print "<EM>$INPUT{'advert'}</EM> Advert:</STRONG></BIG></BIG>\n";
	print "<P><CENTER><TABLE CELLPADDING=3>\n";
	print "<TR ALIGN=CENTER VALIGN=BOTTOM>";
	print "<TD><EM>Date</EM><BR><HR></TD>";
	if ($bannex) {
		print "<TD><EM>Banners Shown</EM><BR><HR></TD>";
	}
	print "<TD><EM>Exposures</EM><BR><HR></TD>";
	print "<TD><EM>Clicks</EM><BR><HR></TD>";
	print "<TD><EM>%</EM><BR><HR></TD>";
	print "<TD><EM>Ratio</EM><BR><HR></TD></TR>\n";
	foreach $daycount ($startday..$endday) {
		print "<TR ALIGN=CENTER>";
		if (($daycount > $startday)
		  && ($daycount-(int($daycount/7)*7)==3)) {
			print "<TD COLSPAN=5><HR WIDTH=50%>";
			print "</TD></TR>\n";
			print "<TR ALIGN=CENTER>";
		}
		&count_to_date($daycount);
		if ($perp_mon < 10) { $perp_mon = "0$perp_mon"; }
		if ($perp_day < 10) { $perp_day = "0$perp_day"; }
		$perp_year = $perp_year + 1900;
		print "<TD>$perp_day $months[$perp_mon-1] $perp_year</TD>";
		$banners = "$perp_day $perp_mon $perp_year S";
		$exposures = "$perp_day $perp_mon $perp_year E";
		$clicks = "$perp_day $perp_mon $perp_year C";
		$banners = $accesses{$banners};
		$exposures = $accesses{$exposures};
		$clicks = $accesses{$clicks};
		if ($banners < 1) { $banners = "0"; }
		if ($exposures < 1) { $exposures = "0"; }
		if ($clicks < 1) { $clicks = "0"; }
		if ($bannex) {
			print "<TD>",&commas($banners),"</TD>";
		}
		print "<TD>",&commas($exposures),"</TD>";
		print "<TD>",&commas($clicks),"</TD>";
		if ($exposures == 0) {
			$perc = "-";
			$ratio = "-";
		}
		elsif ($clicks == 0) {
			$perc = "-";
			$ratio = "-";
		}
		else {
			$perc = ((100*($clicks/$exposures))+.05001);
			$ratio = (($exposures/$clicks)+.5001);
		}
		unless ($perc eq "-") {
			$perc =~ s/(\d+\.\d).*/$1/;
			$perc = $perc."%";
		}
		unless ($ratio eq "-") {
			$ratio =~ s/(\d+)\.\d.*/$1/;
			$ratio = $ratio.":1";
		}
		print "<TD>$perc</TD><TD>$ratio</TD></TR>\n";
	}
	print "</TABLE></CENTER></P>\n";
	print "<P><CENTER><FORM METHOD=POST ACTION=$admin_cgi>\n";
	print "<INPUT TYPE=HIDDEN NAME=password ";
	print "VALUE=$INPUT{'password'}>\n";
	print "<INPUT TYPE=HIDDEN NAME=reviewone VALUE=";
	if ($INPUT{'groupstatus'}) {
		print "\"$INPUT{'groupstatus'}\"";
	}
	else {
		print "$INPUT{'advert'}";
	}
	print ">\n";
	print "<INPUT TYPE=SUBMIT ";
	print "VALUE=\"View Overall Stats\"> ";
	print "</P></FORM></CENTER>\n";
	&Footer;
	exit;
}

sub ShowAdvert {
	@image = split (/\|/,$image);
	foreach $image (@image) {
		print "<P ALIGN=CENTER><IMG SRC=\"$image\"></P>\n";
	}
	if ($url) {
		print "<P ALIGN=CENTER>Destination: ";
		print "<A HREF=\"$url\">$url</A></P>\n";
	}
}

sub edit {
	$INPUT{'editad'} =~ s/[^\w_-]//g;
	if (-e "$adverts_dir/$INPUT{'editad'}.txt") {
		$lockerror = &LockFile("$INPUT{'editad'}.txt");
		if ($lockerror) {
			&Error_File("$INPUT{'editad'}.txt");
			exit;
		}
		open (DISPLAY, "+<$adverts_dir/$INPUT{'editad'}.txt");
		@lines = <DISPLAY>;
		close (DISPLAY);
		&UnlockFile("$INPUT{'editad'}.txt");
		foreach $line (@lines) {
			chop ($line) if ($line =~ /\n$/);
			$line =~ s/&/&amp;/g;
			$line =~ s/>/&gt;/g;
			$line =~ s/</&lt;/g;
			$line =~ s/"/&quot;/g;
		}
		($max,$shown,$visits,$url,$image,$height,$width,
		  $alt,$pass,$text,$start,$weight,$zone,
		  $border,$target,$raw,$displayratio) = @lines;
		($max,$maxtype) = split (/\|/, $max);
		($text,$texttype) = split (/\|/, $text);
		($displayratio,$displaycount) = split (/\|/, $displayratio);
	}
	unless ($maxtype) { $maxtype = "E"; }
	unless ($texttype) { $texttype = "B"; }
	unless ($url) { $url = "http://"; }
	$image =~ s/\|/\n/g;
	unless ($image) { $image = "http://"; }
	unless ($weight) {
		if ($max || $displayratio) { $weight = "0"; }
		else { $weight = 1; }
	}
	unless ($border) {
		if ($max || $displayratio) { $border = "0"; }
		else { $border = 2; }
	}
	if ($target eq "_top") { $target = "TARGET=&quot;_top&quot;"; }
	&Header("WebAdverts","Add/Edit/Delete Advert");
	print "<FORM METHOD=POST ACTION=$admin_cgi>\n";
	print "<P ALIGN=CENTER><BIG><STRONG>Info for the ";
	print "<EM>$INPUT{'editad'}</EM> Advert:";
	print "</STRONG></BIG>\n";
	print "<P><CENTER><TABLE CELLPADDING=3>\n";
	print "<TR><TD COLSPAN=2><HR WIDTH=50%></TD></TR>\n";
	print "<TR><TD COLSPAN=2>";
	print "<STRONG>I. General Information</STRONG>: ";
	print "The following information is needed for each advert, ";
	print "no matter <EM>how</EM> you're setting things up. Note ";
	print "that while <EM>either</EM> an expiration or a display ";
	print "ratio must be set, it is not necessary to set both. ";
	print "If you're running a &quot;banner exchange,&quot; ";
	print "the display ratio will define how many banner displays ";
	print "earn one exposure. If you're running straight adverts, ";
	print "leave it set to 0, and simply set the appropriate ";
	print "expiration criteria instead. (If you're running ";
	print "an exchange, but want a member to get extra ";
	print "&quot;bonus&quot; exposures, set that bonus number ";
	print "in the &quot;expiration&quot; slot.)</TD></TR>\n";
	if ($maxtype eq "D") {
		$max = int((($max-$time)/86400)+.5);
		if ($max < 0) { $max = 0; }
	}
	print "<TR><TD>Expiration:";
	print "</TD><TD><INPUT TYPE=TEXT ";
	print "NAME=purch VALUE=\"$max\" SIZE=25>";
	print "<BR><INPUT TYPE=RADIO NAME=purchtype VALUE=E";
	if ($maxtype eq "E") { print " CHECKED"; }
	print "> Exposures <INPUT TYPE=RADIO NAME=purchtype VALUE=C";
	if ($maxtype eq "C") { print " CHECKED"; }
	print "> Clicks <INPUT TYPE=RADIO NAME=purchtype VALUE=D";
	if ($maxtype eq "D") { print " CHECKED"; }
	print "> Days</TD></TR>\n";
	print "<TR><TD>Display Ratio:</TD><TD><INPUT TYPE=TEXT ";
	print "NAME=displayratio VALUE=\"$displayratio\" SIZE=5>";
	print "</TD></TR>\n";
	print "<TR><TD>Site URL:</TD><TD><INPUT TYPE=TEXT ";
	print "NAME=url VALUE=\"$url\" SIZE=50></TD></TR>\n";
	print "<TR><TD>Banner URL(s):</TD><TD>";
	print "<TEXTAREA COLS=50 ROWS=5 NAME=image WRAP=VIRTUAL>";
	print "$image</TEXTAREA></TD></TR>\n";
	print "<TR><TD>Weight (Wt.):</TD><TD><INPUT TYPE=TEXT ";
	print "NAME=weight VALUE=\"$weight\" SIZE=5>";
	print "<BR><EM>(0 = never; 1 = always; ";
	print "higher numbers = less often)</EM>";
	print "</TD></TR>\n";
	print "<TR><TD>Zone(s):</TD><TD><INPUT TYPE=TEXT ";
	print "NAME=zone VALUE=\"$zone\" SIZE=25></TD></TR>\n";
	print "<TR><TD>Password:</TD><TD><INPUT TYPE=TEXT ";
	print "NAME=pass VALUE=\"$pass\" SIZE=50></TD></TR>\n";
	print "<TR><TD COLSPAN=2><HR WIDTH=50%></TD></TR>\n";
	print "<TR><TD COLSPAN=2>";
	print "<STRONG>II. SSI Advert Information</STRONG>: ";
	print "The following information is used to define or modify ";
	print "the &quot;appearance&quot; of an advert banner when ";
	print "called via an SSI tag. None of it is necessary ";
	print "(or even useful) if the advert is called only via IMG ";
	print "tags.</TD></TR>\n";
	print "<TR><TD>Link Attributes:</TD><TD><INPUT TYPE=TEXT ";
	print "NAME=target VALUE=\"$target\" SIZE=50></TD></TR>\n";
	print "<TR><TD>Banner Height:</TD><TD><INPUT TYPE=TEXT ";
	print "NAME=height VALUE=\"$height\" SIZE=5> pixels</TD></TR>\n";
	print "<TR><TD>Banner Width:</TD><TD><INPUT TYPE=TEXT ";
	print "NAME=width VALUE=\"$width\" SIZE=5> pixels</TD></TR>\n";
	print "<TR><TD>Border:</TD><TD><INPUT TYPE=TEXT ";
	print "NAME=border VALUE=\"$border\" SIZE=5></TD></TR>\n";
	print "<TR><TD>ALT Text:</TD><TD><INPUT TYPE=TEXT ";
	print "NAME=alt VALUE=\"$alt\" SIZE=50></TD></TR>\n";
	print "<TR><TD>Link Text:</TD><TD><INPUT TYPE=TEXT ";
	print "NAME=text VALUE=\"$text\" SIZE=50>";
	print "<BR><INPUT TYPE=RADIO NAME=texttype VALUE=T";
	if ($texttype eq "T") { print " CHECKED"; }
	print "> Above Banner <INPUT TYPE=RADIO NAME=texttype VALUE=B";
	if ($texttype eq "B") { print " CHECKED"; }
	print "> Below Banner</TD></TR>\n";
	print "<TR><TD COLSPAN=2><HR WIDTH=50%></TD></TR>\n";
	print "<TR><TD COLSPAN=2>";
	print "<STRONG>II. &quot;Raw Mode&quot; Information</STRONG>: ";
	print "If you so choose, you can specify below <EM>exactly</EM> ";
	print "how an advert is to appear on your pages. (Again, of ";
	print "course, this is only possible if you're using SSI tags; ";
	print "&quot;raw mode&quot; and non-SSI display are ";
	print "incompatible.) <EM>Only use this option if you're sure ";
	print "you know what you're doing; anything input here will ";
	print "override just about everything above!</EM></TD></TR>\n";
	print "<TR><TD>\"Raw\" HTML:</TD><TD>";
	print "<TEXTAREA COLS=50 ROWS=5 NAME=raw WRAP=VIRTUAL>";
	print "$raw</TEXTAREA></TD></TR>\n";
	print "<TR><TD COLSPAN=2><HR WIDTH=50%></TD></TR>\n";
	print "</TABLE></P>\n";
	print "<P><INPUT TYPE=HIDDEN NAME=editad ";
	print "VALUE=\"$INPUT{'editad'}\">\n";
	print "<INPUT TYPE=HIDDEN NAME=start VALUE=\"$start\">\n";
	print "Check here to reset advert start date, ";
	print "exposures &amp; clicks: ";
	print "<INPUT TYPE=CHECKBOX NAME=\"resetadvert\">";
	print "<P><INPUT TYPE=SUBMIT NAME=editfinal ";
	print "VALUE=\"Add/Edit Advert\">\n";
	if (-e "$adverts_dir/$INPUT{'editad'}.txt") {
		print "<P><INPUT TYPE=SUBMIT NAME=del ";
		print "VALUE=\"Delete Advert\"> ";
		print "<INPUT TYPE=HIDDEN NAME=delad ";
		print "VALUE=\"$INPUT{'editad'}\">\n";
	}
	print "</P></CENTER></FORM>\n";
	&Footer;
	exit;
}

sub groupedit {
	$INPUT{'editgroup'} =~ s/[^\w_]//g;
	if (-e "$adverts_dir/$INPUT{'editgroup'}.grp") {
		$lockerror = &LockFile("$INPUT{'editgroup'}.grp");
		if ($lockerror) {
			&Error_File("$INPUT{'editgroup'}.grp");
			exit;
		}
		open (DISPLAY, "+<$adverts_dir/$INPUT{'editgroup'}.grp");
		@lines = <DISPLAY>;
		close (DISPLAY);
		&UnlockFile("$INPUT{'editgroup'}.grp");
		$password = @lines[0];
		$adverts = join(' ',@lines);
	}
	&Header("WebAdverts","Add/Edit/Delete Group");
	print "<FORM METHOD=POST ACTION=$admin_cgi>\n";
	print "<P><CENTER><BIG><STRONG>Info for the ";
	print "<EM>$INPUT{'editgroup'}</EM> Group:";
	print "</STRONG></BIG>\n";
	print "<P>Select the adverts to be included in this group:\n";
	$lockerror = &LockFile(adcount.txt);
	if ($lockerror) {
		undef (@lines);
	}
	else {
		open (COUNT, "+<$adverts_dir/adcount.txt");
		@lines = <COUNT>;
		close (COUNT);
		&UnlockFile("adcount.txt");
	}
	foreach $line (@lines) { chop ($line) if ($line =~ /\n$/); }
	$max = @lines - 1;
	@advertisements = @lines[3..$max];
	$size = @advertisements;
	print "<P><SELECT NAME=groupadverts MULTIPLE ";
	print "SIZE=$size>\n";
	foreach $advertiser (@advertisements) {
		next if (length($advertiser) < 1);
		print "<OPTION VALUE=\"$advertiser\"";
		if ($adverts && ($adverts =~ $advertiser)) {
			print " SELECTED";
		}
		print "> $advertiser ";
	}
	print "</SELECT>\n";
	print "<P>Password: <INPUT TYPE=TEXT ";
	print "NAME=pass VALUE=\"$password\" SIZE=25>\n";
	print "<P><INPUT TYPE=HIDDEN NAME=editgroup ";
	print "VALUE=\"$INPUT{'editgroup'}\">\n";
	print "<INPUT TYPE=SUBMIT NAME=editgroupfinal ";
	print "VALUE=\"Add/Edit Group\">\n";
	if (-e "$adverts_dir/$INPUT{'editgroup'}.grp") {
		print "<P><INPUT TYPE=SUBMIT NAME=delgroup ";
		print "VALUE=\"Delete Group\"> ";
		print "<INPUT TYPE=HIDDEN NAME=delgroupname ";
		print "VALUE=\"$INPUT{'editgroup'}\">\n";
	}
	print "</P></CENTER></FORM>\n";
	&Footer;
	exit;
}

sub del {
	unless (-e "$adverts_dir/$INPUT{'delad'}.txt") {
		&Header("WebAdverts Error!","Invalid Advert Name!");
		print "<P ALIGN=CENTER>There is no advert with the name ";
		print "<STRONG>&quot;$INPUT{'delad'}&quot;</STRONG> ";
		print "on the list!\n";
		print "<P ALIGN=CENTER>(Note that advert names ";
		print "<EM>are</EM> case sensitive!)</P>\n";
		&Footer;
		exit;
	}
	&Header("WebAdverts","&quot;Delete&quot; Request Confirmation");
	print "<FORM METHOD=POST ACTION=$admin_cgi>\n";
	print "<P><CENTER>Are you <EM>sure</EM> you want to delete the ";
	print "<STRONG>$INPUT{'delad'}</STRONG> advert?\n";
	print "<INPUT TYPE=HIDDEN NAME=delad VALUE=$INPUT{'delad'}>\n";
	print "<INPUT TYPE=SUBMIT NAME=delfinal VALUE=\"Yes\">\n";
	print "</CENTER></P></FORM>\n";
	&Footer;
	exit;
}

sub delgroup {
	unless (-e "$adverts_dir/$INPUT{'delgroupname'}.grp") {
		&Header("WebAdverts Error!","Invalid Group Name!");
		print "<P ALIGN=CENTER>There is no group ";
		print "on the list defined with the name <STRONG>";
		print "&quot;$INPUT{'delgroupname'}&quot;</STRONG>!\n";
		print "<P ALIGN=CENTER>(Note that group names ";
		print "<EM>are</EM> case sensitive!)</P>\n";
		&Footer;
		exit;
	}
	&Header("WebAdverts","&quot;Delete&quot; Request Confirmation");
	print "<FORM METHOD=POST ACTION=$admin_cgi>\n";
	print "<P><CENTER>Are you <EM>sure</EM> you want to delete the ";
	print "<STRONG>$INPUT{'delgroupname'}</STRONG> group? ";
	print "<INPUT TYPE=HIDDEN NAME=delgroupname ";
	print "VALUE=$INPUT{'delgroupname'}>\n";
	print "<INPUT TYPE=SUBMIT NAME=delgroupfinal VALUE=\"Yes\">\n";
	print "</CENTER></FORM>\n";
	print "<P>(Please note that deleting the group will ";
	print "<EM>not</EM> delete or otherwise affect the adverts ";
	print "themselves. Only the ability to view all their stats ";
	print "on a single page will be gone!)</P>\n";
	&Footer;
	exit;
}

sub newpass {
	unless ($INPUT{'passad'} eq $INPUT{'passad2'}) {
		&Header("WebAdverts Error!","Password Mismatch!");
		print "<P ALIGN=CENTER>Your administrative password was ";
		print "not set, as the two entries were different!</P>\n";
		&Footer;
		exit;
	}
	$lockerror = &LockFile("adpassword.txt");
	if ($lockerror) {
		&Error_File("adpassword.txt");
		exit;
	}
	open (PASSWORD, "$adverts_dir/adpassword.txt");
	$password = <PASSWORD>;
	close (PASSWORD);
	&UnlockFile("adpassword.txt");
	chop ($password) if ($password =~ /\n$/);
	if ($password) {
		if ($INPUT{'cryptword'}) {
			$newpassword = $INPUT{'cryptword'};
		}
		elsif ($INPUT{'password'}) {
			$newpassword = crypt($INPUT{'password'}, aa);
		}
		else {
			&Header("WebAdverts Error!","No Password!");
			print "<P ALIGN=CENTER>You must ";
			print "enter a password!</P>\n";
			&Footer;
			exit;
		}
		unless ($newpassword eq $password) {
			&Header("WebAdverts Error!","Invalid Password!");
			print "<P ALIGN=CENTER>";
			print "The password you entered is incorrect!</P>\n";
			&Footer;
			exit;
		}
	}
	$newpassword = crypt($INPUT{'passad'}, aa);
	$lockerror = &LockFile("adpassword.txt");
	if ($lockerror) {
		&Error_File("adpassword.txt");
		exit;
	}
	open (PASSWORD, ">$adverts_dir/adpassword.txt");
	print PASSWORD "$newpassword";
	close (PASSWORD);
	&UnlockFile("adpassword.txt");
	&Header("WebAdverts","Password Set");
	print "<P ALIGN=CENTER>Your administrative password ";
	print "has been set.</P>\n";
	&LinkBack;
	&Footer;
	exit;
}

sub resetcount {
	$lockerror = &LockFile("adcount.txt");
	if ($lockerror) {
		&Error_File("adcount.txt");
		exit;
	}
	open (COUNT, "+<$adverts_dir/adcount.txt");
	@lines = <COUNT>;
	foreach $line (@lines) { chop ($line) if ($line =~ /\n$/); }
	$max = @lines - 1;
	seek(COUNT, 0, 0);
	print COUNT "1\n";
	print COUNT "0\n";
	print COUNT "$time\n";
	@advertisements = @lines[3..$max];
	foreach $advertiser (@advertisements) {
		next if (length($advertiser) < 1);
		print COUNT "$advertiser\n";
	}
	truncate (COUNT, tell(COUNT));
	close (COUNT);
	&UnlockFile("adcount.txt");
	$actionresult = 1;
	&reviewall;
}

sub editfinal {
	$editad = $INPUT{'editad'};
	$INPUT{'url'} =~ s/\s//g;
	unless ($INPUT{'url'} =~ /;|\*|@|(\.\.)|(^\.)|(\/\/\.)/ ||
	  $INPUT{'url'} !~ /.*\:\/\/.*\..*/) {
		$url = $INPUT{'url'};
	}
	@image = split (/\cM|\n/,$INPUT{'image'});
	foreach $fauximage (@image) {
		$fauximage =~ s/\s//g;
		unless ($fauximage =~ /;|\*|@|(\.\.)|(^\.)|(\/\/\.)/ ||
		  $fauximage !~ /.*\:\/\/.*\..*/) {
			$image = $image.$fauximage."|";
		}
	}
	chop $image;
	if ($INPUT{'purchtype'} eq "D") {
		$INPUT{'purch'} = $time + ($INPUT{'purch'}*86400);
	}
	$INPUT{'purch'} =~ s/[^\d]//g;
	$max = $INPUT{'purch'};
	$maxtype = $INPUT{'purchtype'};
	$displayratio = $INPUT{'displayratio'};
	$height = int($INPUT{'height'});
	$width = int($INPUT{'width'});
	$alt = $INPUT{'alt'};
	$INPUT{'text'} =~ s/^\s+//;
	$INPUT{'text'} =~ s/\s+$//;
	$INPUT{'text'} =~ s/\s+/ /g;
	$text = $INPUT{'text'};
	$texttype = $INPUT{'texttype'};
	$pass = $INPUT{'pass'};
	$weight = int($INPUT{'weight'});
	$zone = $INPUT{'zone'};
	$zone =~ s/^\s+//;
	$zone =~ s/\s+$//;
	$zone =~ s/\s+/ /g;
	$border = int($INPUT{'border'});
	$target = $INPUT{'target'};
	$INPUT{'raw'} =~ s/\cM|\n/ /g;
	$INPUT{'raw'} =~ s/^\s+//;
	$INPUT{'raw'} =~ s/\s+$//;
	$INPUT{'raw'} =~ s/\s+/ /g;
	$raw = $INPUT{'raw'};
	unless (($max || $displayratio) && $pass && ($raw || $image)) {
		&Header("WebAdverts Error!","Incomplete Advert!");
		print "<P>You didn't provide all of the necessary ";
		print "information (expiration criteria, banner URL ";
		print "and password) to allow addition of the ";
		print "<STRONG>$editad</STRONG> advert!</P>\n";
		&Footer;
		exit;
	}
	if (-e "$adverts_dir/$editad.txt") {
		$lockerror = &LockFile("$editad.txt");
		if ($lockerror) {
			&Error_File("$editad.txt");
			exit;
		}
		open (DISPLAY, "+<$adverts_dir/$editad.txt");
		@lines = <DISPLAY>;
		close (DISPLAY);
		&UnlockFile("$editad.txt");
		foreach $line (@lines) { chop ($line) if ($line =~ /\n$/); }
		($shown,$visits,$start,$displaycount) = @lines[1,2,10,16];
		($other,$displaycount) = split (/\|/, $displaycount);
	}
	$lockerror = &LockFile("adcount.txt");
	if ($lockerror) {
		&Error_File("adcount.txt");
		exit;
	}
	open (COUNT, "+<$adverts_dir/adcount.txt");
	@lines = <COUNT>;
	close (COUNT);
	&UnlockFile("adcount.txt");
	foreach $line (@lines) {
		chop ($line) if ($line =~ /\n$/);
		if ($line eq $editad) { $PresenceCheck = 1; }
	}
	if ($INPUT{'resetadvert'} || (!$PresenceCheck && !$shown)) {
		$shown = 0;
		$visits = 0;
		$start = time;
		$displaycount = 0;
		unlink ("$adverts_dir/$editad.log");
		unlink ("$adverts_dir/$editad.log.bak");
	}
	$lockerror = &LockFile("$editad.txt");
	if ($lockerror) {
		&Error_File("$editad.txt");
		exit;
	}
	open (DISPLAY, ">$adverts_dir/$editad.txt");
	print DISPLAY "$max|$maxtype\n";
	print DISPLAY "$shown\n";
	print DISPLAY "$visits\n";
	print DISPLAY "$url\n";
	print DISPLAY "$image\n";
	print DISPLAY "$height\n";
	print DISPLAY "$width\n";
	print DISPLAY "$alt\n";
	print DISPLAY "$pass\n";
	print DISPLAY "$text|$texttype\n";
	print DISPLAY "$start\n";
	print DISPLAY "$weight\n";
	print DISPLAY " $zone \n";
	print DISPLAY "$border\n";
	print DISPLAY "$target\n";
	print DISPLAY "$raw\n";
	print DISPLAY "$displayratio|$displaycount\n";
	close (DISPLAY);
	&UnlockFile("$editad.txt");
	unless ($PresenceCheck) {
		$lockerror = &LockFile("adcount.txt");
		if ($lockerror) {
			&Error_File("adcount.txt");
			exit;
		}
		open (COUNT, ">>$adverts_dir/adcount.txt");
		print COUNT "$editad\n";
		close (COUNT);
		&UnlockFile("adcount.txt");
	}
	&Header("WebAdverts","Advert Updated!");
	print "<P ALIGN=CENTER><STRONG>$editad</STRONG>\n";
	$name = $editad;
	&ShowAdvert;
	if ($displayratio > 0) {
		print "<P>This advert earns an exposure for each ";
		print "<STRONG>$displayratio</STRONG> display";
		if ($displayratio > 1) { print "s"; }
		print ". The HTML code on the advertiser's pages should ";
		print "look similar to the following.\n";
		print "<P><STRONG>&lt;P ALIGN=CENTER&gt;&lt;A ";
		print "HREF=&quot;$nonssi_cgi?advert=NonSSI&quot;&gt;";
		print "&lt;IMG SRC=&quot;$nonssi_cgi?";
		print "ID=$name&quot;&gt;&lt;/A&gt;</STRONG>";
	}
	&LinkBack;
	&Footer;
	exit;
}

sub editgroupfinal {
	$editgroup = $INPUT{'editgroup'};
	$pass = $INPUT{'pass'};
	@groupadverts = split(',',$INPUT{'groupadverts'});
	unless ($pass && (@groupadverts > 0)) {
		&Header("WebAdverts Error!","Incomplete Form!");
		print "<P>You didn't provide all of the necessary ";
		print "information to allow creation ";
		print "of the <STRONG>$editgroup</STRONG> ";
		print "group!</P>\n";
		&Footer;
		exit;
	}
	$lockerror = &LockFile("$editgroup.grp");
	if ($lockerror) {
		&Error_File("$editgroup.grp");
		exit;
	}
	open (GROUP, ">$adverts_dir/$editgroup.grp");
	print GROUP "$pass\n";
	foreach $advert (@groupadverts) {
		print GROUP "$advert\n";
	}
	close (GROUP);
	&UnlockFile("$editgroup.grp");
	if (-e "$adverts_dir/groups.txt") {
		$lockerror = &LockFile("groups.txt");
		if ($lockerror) {
			&Error_File("groups.txt");
			exit;
		}
		open (COUNT, "+<$adverts_dir/groups.txt");
		@lines = <COUNT>;
		close (COUNT);
		&UnlockFile("groups.txt");
	}
	foreach $line (@lines) {
		chop ($line) if ($line =~ /\n$/);
		if ($line eq $editgroup) { $PresenceCheck = 1; }
	}
	unless ($PresenceCheck) {
		$lockerror = &LockFile("groups.txt");
		if ($lockerror) {
			&Error_File("groups.txt");
			exit;
		}
		open (COUNT, ">>$adverts_dir/groups.txt");
		print COUNT "$editgroup\n";
		close (COUNT);
		&UnlockFile("groups.txt");
	}
	&Header("WebAdverts","Group Updated!");
	print "<P ALIGN=CENTER>";
	print "The <STRONG>$editgroup</STRONG> advert now includes ";
	print "the following adverts:\n";
	print "<P ALIGN=CENTER><STRONG>";
	foreach $advert (@groupadverts) {
		chop $advert if ($advert =~ /\n$/);
		print "$advert ";
	}
	print "</STRONG>\n";
	&LinkBack;
	&Footer;
	exit;
}

sub delfinal {
	$delad = $INPUT{'delad'};
	unlink ("$adverts_dir/$delad.txt");
	unlink ("$adverts_dir/$delad.txt.bak");
	unlink ("$adverts_dir/$delad.log");
	unlink ("$adverts_dir/$delad.log.bak");
	$lockerror = &LockFile("adcount.txt");
	if ($lockerror) {
		&Error_File("adcount.txt");
		exit;
	}
	open (COUNT, "+<$adverts_dir/adcount.txt");
	@lines = <COUNT>;
	foreach $line (@lines) { chop ($line) if ($line =~ /\n$/); }
	seek(COUNT, 0, 0);
	foreach $line (@lines) {
		unless (($line eq $delad) || (length($line) < 1)) {
			print COUNT "$line\n";
		}
	}
	truncate (COUNT, tell(COUNT));
	close (COUNT);
	&UnlockFile("adcount.txt");
	$actionresult = 1;
	&reviewall;
}

sub delgroupfinal {
	$delgroup = $INPUT{'delgroupname'};
	unlink ("$adverts_dir/$delgroup.grp");
	$lockerror = &LockFile("groups.txt");
	if ($lockerror) {
		&Error_File("groups.txt");
		exit;
	}
	open (COUNT, "+<$adverts_dir/groups.txt");
	@lines = <COUNT>;
	foreach $line (@lines) { chop ($line) if ($line =~ /\n$/); }
	seek(COUNT, 0, 0);
	foreach $line (@lines) {
		unless (($line eq $delgroup) || (length($line) < 1)) {
			print COUNT "$line\n";
		}
	}
	truncate (COUNT, tell(COUNT));
	close (COUNT);
	&UnlockFile("groups.txt");
	$actionresult = 1;
	&reviewall;
}

sub LinkBack {
	$lockerror = &LockFile("adpassword.txt");
	if ($lockerror) {
		&Error_File("adpassword.txt");
		exit;
	}
	open (PASSWORD, "$adverts_dir/adpassword.txt");
	$password = <PASSWORD>;
	close (PASSWORD);
	&UnlockFile("adpassword.txt");
	chop ($password) if ($password =~ /\n$/);
	print "<P><CENTER>\n";
	print "<FORM METHOD=POST ACTION=$admin_cgi>\n";
	print "<INPUT TYPE=HIDDEN NAME=cryptword ";
	print "VALUE=\"$password\">\n";
	print "<INPUT TYPE=HIDDEN NAME=reviewone ";
	print "VALUE=\"Review All Adverts\">\n";
	print "<INPUT TYPE=SUBMIT ";
	print "VALUE=\"Reload Advert Index\">\n";
	print "</FORM></CENTER></P>\n";
}

sub Header {
	($title,$header) = @_;
	print "<HTML><HEAD><TITLE>$title</TITLE></HEAD>\n";
	print "<BODY BGCOLOR=\"#ccffff\" TEXT=\"#000000\">\n";
	print "<HR><H1 ALIGN=CENTER>$header</H1><HR>\n";
}

sub Footer {
	print "<HR><P ALIGN=CENTER><SMALL>";
	print "Maintained with <STRONG>";
	print "<A HREF=\"http://awsd.com/scripts/webadverts/\">";
	print "WebAdverts $version</A></STRONG>.</SMALL></P>\n";
	print "</BODY></HTML>\n";
}

sub register {
	unless ($INPUT{'register'} eq "frtytw") {
		&Header("WebAdverts Error!","Incorrect Code!");
		print "<P ALIGN=CENTER>Sorry, but the registration code ";
		print "you entered is incorrect!</P>\n";
		&Footer;
		exit;
	}
	open (REGISTER, ">$adverts_dir/register.txt");
	print REGISTER "frtytw";
	close (REGISTER);
	&Header("WebAdverts","Thanks For Registering!");
	print "<P ALIGN=CENTER>Your support is appreciated!</P>\n";
	&LinkBack;
	&Footer;
	exit;
}

sub reginfo {
	&Header("WebAdverts","Registration Information");
	print "<P>WebAdverts is distributed as shareware. While you ";
	print "are free to modify and use it as you see fit, any ";
	print "commercial use (i.e., any use which generates revenue) ";
	print "should be registered. The registration fee is just \$50 ";
	print "(US). Payment should be sent via check or money order ";
	print "to <STRONG>Darryl C. Burgdorf, Affordable Web Space ";
	print "Design, 3524 Pacific Street, Omaha NE 68105</STRONG>.\n";
	print "<P>(If you happen to live in a country other than the ";
	print "United States, you can write a check in your local ";
	print "currency for the equivalent of \$57.50. That will cover ";
	print "the \$50 registration fee and the \$7.50 ";
	print "service fee which my bank charges. Please do ";
	print "<STRONG><EM>not</EM></STRONG> write me a check ";
	print "in US funds drawn on a non-US bank; the service charge ";
	print "for those can be anywhere from \$10 to \$25!)\n";
	print "<P>Thank you for your support!\n";
	print "<P><CENTER>\n";
	print "<FORM METHOD=POST ACTION=$admin_cgi>\n";
	print "<INPUT TYPE=SUBMIT VALUE=\"Enter Registration Code:\">\n";
	print "<INPUT TYPE=TEXT NAME=register SIZE=10>\n";
	print "</FORM></CENTER></P>\n";
	&Footer;
	exit;
}

sub date_to_count {
	($perp_mon,$perp_day,$perp_year) = @_;
	%day_counts =
	  (1,0,2,31,3,59,4,90,5,120,6,151,7,181,
	  8,212,9,243,10,273,11,304,12,334);
	$perp_days = (($perp_year-93)*365)+(int(($perp_year-93)/4));
	$perp_days = $perp_days + $day_counts{$perp_mon};
	if ((int(($perp_year-92)/4) eq (($perp_year-92)/4))
	  && ($perp_mon>2)) {
		$perp_days++;
	}
	$perp_days = $perp_days + $perp_day;
}

sub count_to_date {
	local($perp_days) = @_;
	%day_counts =
	  (1,0,2,31,3,59,4,90,5,120,6,151,
	  7,181,8,212,9,243,10,273,11,304,12,334);
	$perp_year = (int(($perp_days-1)/1461))*4;
	$perp_days = $perp_days-(int(($perp_days-1)/1461)*1461);
	if ($perp_days == 1461) {
		$perp_year = 93+$perp_year+3;
		$perp_days = $perp_days-1095;
	}
	else {
		$perp_year = 93+$perp_year+(int(($perp_days-1)/365));
		$perp_days = $perp_days-(int(($perp_days-1)/365)*365);
	}
	foreach $key (sort {$a <=> $b} keys %day_counts) {
		$perp_count = $day_counts{$key};
		if ((int(($perp_year-92)/4) eq (($perp_year-92)/4))
		  && ($key>2)) {
			$perp_count++;
		}
		if ($perp_days > $perp_count) {
			$perp_mon = $key;
			$perp_subtract = $perp_count;
		}
	}
	$perp_day = $perp_days-$perp_subtract;
}

sub LockFile {
	local($filename) = $_[0];
	local(*FILE) = $_[0];
	local($TrysLeft) = 10;
	if ((-M "$adverts_dir/$filename.lok") > .001) {
		unlink ("$adverts_dir/$filename.lok");
	}
	while ($TrysLeft--) {
		if (-f "$adverts_dir/$filename.lok") {
			select(undef,undef,undef,0.1);
		}
		else {
			open (FILE,">$adverts_dir/$filename.lok");
			if ($UseLocking) {
				$lockresult = eval("flock(FILE,6)");
				if ($@) {
					$UseLocking = 0;
					last;
				}
				if (!$lockresult) {
					close (FILE);
					unlink ("$adverts_dir/$filename.lok");
					select(undef,undef,undef,0.1);
				}
				else {
					last;
				}
			}
			else {
				last;
			}
		}
	}
	if ($TrysLeft >= 0) {
		if ($UseBackup) {
			if (((stat("$adverts_dir/$filename"))[7]) > 20) {
				rename ("$adverts_dir/$filename",
				  "$adverts_dir/$filename.bak");
			}
			if (((stat("$adverts_dir/$filename.bak"))[7]) > 20) {
				system ("cp $adverts_dir/$filename.bak $adverts_dir/$filename");
			}
			system ("chmod a+r $adverts_dir/$filename $adverts_dir/$filename.bak");
		}
		return 0;
	}
	else {
		return 1;
	}
}

sub UnlockFile {
	local($filename) = $_[0];
	local(*FILE) = $_[0];
	close (FILE);
	unlink ("$adverts_dir/$filename.lok");
}

sub commas {
	local($_)=@_;
	1 while s/(.*\d)(\d\d\d)/$1,$2/;
	$_;
}

sub Error_File {
	&Header("WebAdverts Error!","File Permission Error!");
	print "<P>The server encountered an error while trying ";
	print "to access <STRONG>$_[0]</STRONG>!\n";
	print "<P>The most likely cause of the problem is a permissions ";
	print "error in your adverts directory ($adverts_dir). Make ";
	print "sure that the directory exists and that it is set ";
	print "world-writable.</P>\n";
	&Footer;
	exit;
}

